@@ -121,7 +121,7 @@ async fn test_auto_probe() {
                 assert!(peer.is_none());
                 assert_eq!(error, OutboundProbeError::NoAddresses);
             }
-            other => panic!("Unexpected Event: {:?}", other),
+            other => panic!("Unexpected behaviour event: {:?}.", other),
         }
 
         assert_eq!(client.behaviour().nat_status(), NatStatus::Unknown);
@@ -139,7 +139,7 @@ async fn test_auto_probe() {
                 assert_eq!(peer, server_id);
                 probe_id
             }
-            other => panic!("Unexpected Event: {:?}", other),
+            other => panic!("Unexpected behaviour event: {:?}.", other),
         };
 
         match next_event(&mut client).await {
@@ -155,7 +155,7 @@ async fn test_auto_probe() {
                     OutboundProbeError::Response(ResponseError::DialError)
                 );
             }
-            other => panic!("Unexpected Event: {:?}", other),
+            other => panic!("Unexpected behaviour event: {:?}.", other),
         }
 
         match next_event(&mut client).await {
@@ -163,7 +163,7 @@ async fn test_auto_probe() {
                 assert_eq!(old, NatStatus::Unknown);
                 assert_eq!(new, NatStatus::Private);
             }
-            other => panic!("Unexpected Event: {:?}", other),
+            other => panic!("Unexpected behaviour event: {:?}.", other),
         }
 
         assert_eq!(client.behaviour().confidence(), 0);
@@ -186,9 +186,10 @@ async fn test_auto_probe() {
                 assert_eq!(peer, server_id);
                 probe_id
             }
-            other => panic!("Unexpected Event: {:?}", other),
+            other => panic!("Unexpected behaviour event: {:?}.", other),
         };
 
+        let mut has_received_response = false;
         // Expect inbound dial from server.
         loop {
             match client.select_next_some().await {
@@ -198,17 +199,34 @@ async fn test_auto_probe() {
                     assert_eq!(peer_id, server_id);
                     break;
                 }
-                SwarmEvent::IncomingConnection { .. } | SwarmEvent::NewListenAddr { .. } => {}
-                _ => panic!("Unexpected Swarm Event"),
+                SwarmEvent::Behaviour(Event::OutboundProbe(OutboundProbeEvent::Response {
+                    probe_id,
+                    peer,
+                    ..
+                })) => {
+                    assert_eq!(peer, server_id);
+                    assert_eq!(probe_id, id);
+                    has_received_response = true;
+                }
+                SwarmEvent::IncomingConnection { .. }
+                | SwarmEvent::NewListenAddr { .. }
+                | SwarmEvent::ExpiredListenAddr { .. } => {}
+                other => panic!("Unexpected swarm event: {:?}.", other),
             }
         }
 
-        match next_event(&mut client).await {
-            Event::OutboundProbe(OutboundProbeEvent::Response { probe_id, peer, .. }) => {
-                assert_eq!(peer, server_id);
-                assert_eq!(probe_id, id);
+        if !has_received_response {
+            match client.select_next_some().await {
+                SwarmEvent::Behaviour(Event::OutboundProbe(OutboundProbeEvent::Response {
+                    probe_id,
+                    peer,
+                    ..
+                })) => {
+                    assert_eq!(peer, server_id);
+                    assert_eq!(probe_id, id);
+                }
+                other => panic!("Unexpected swarm event: {:?}.", other),
             }
-            other => panic!("Unexpected Event: {:?}", other),
         }
 
         // Expect to flip status to public
@@ -218,7 +236,7 @@ async fn test_auto_probe() {
                 assert!(matches!(new, NatStatus::Public(_)));
                 assert!(new.is_public());
             }
-            other => panic!("Unexpected Event: {:?}", other),
+            other => panic!("Unexpected behaviour event: {:?}.", other),
         }
 
         assert_eq!(client.behaviour().confidence(), 0);
@@ -271,7 +289,7 @@ async fn test_confidence() {
                     assert_eq!(peer, server_id);
                     probe_id
                 }
-                other => panic!("Unexpected Event: {:?}", other),
+                other => panic!("Unexpected behaviour event: {:?}.", other),
             };
 
             match next_event(&mut client).await {
@@ -296,7 +314,7 @@ async fn test_confidence() {
                     assert_eq!(peer, server_id);
                     assert_eq!(probe_id, id);
                 }
-                other => panic!("Unexpected Event: {:?}", other),
+                other => panic!("Unexpected behaviour event: {:?}.", other),
             }
 
             // Confidence should increase each iteration up to MAX_CONFIDENCE
@@ -315,7 +333,7 @@ async fn test_confidence() {
                         assert_eq!(old, NatStatus::Unknown);
                         assert_eq!(new.is_public(), test_public);
                     }
-                    other => panic!("Unexpected Event: {:?}", other),
+                    other => panic!("Unexpected behaviour event: {:?}.", other),
                 }
             }
         }
@@ -364,7 +382,7 @@ async fn test_throttle_server_period() {
                 }
                 Event::OutboundProbe(OutboundProbeEvent::Request { .. }) => {}
                 Event::OutboundProbe(OutboundProbeEvent::Response { .. }) => {}
-                other => panic!("Unexpected Event: {:?}", other),
+                other => panic!("Unexpected behaviour event: {:?}.", other),
             }
         }
 
@@ -377,7 +395,7 @@ async fn test_throttle_server_period() {
                 assert!(peer.is_none());
                 assert_eq!(error, OutboundProbeError::NoServer);
             }
-            other => panic!("Unexpected Event: {:?}", other),
+            other => panic!("Unexpected behaviour event: {:?}.", other),
         }
         assert_eq!(client.behaviour().confidence(), 0);
 
@@ -423,14 +441,14 @@ async fn test_use_connected_as_server() {
             Event::OutboundProbe(OutboundProbeEvent::Request { peer, .. }) => {
                 assert_eq!(peer, server_id);
             }
-            other => panic!("Unexpected Event: {:?}", other),
+            other => panic!("Unexpected behaviour event: {:?}.", other),
         }
 
         match next_event(&mut client).await {
             Event::OutboundProbe(OutboundProbeEvent::Response { peer, .. }) => {
                 assert_eq!(peer, server_id);
             }
-            other => panic!("Unexpected Event: {:?}", other),
+            other => panic!("Unexpected behaviour event: {:?}.", other),
         }
 
         drop(_handle);
@@ -481,7 +499,7 @@ async fn test_outbound_failure() {
                 }
                 Event::OutboundProbe(OutboundProbeEvent::Request { .. }) => {}
                 Event::OutboundProbe(OutboundProbeEvent::Response { .. }) => {}
-                other => panic!("Unexpected Event: {:?}", other),
+                other => panic!("Unexpected behaviour event: {:?}.", other),
             }
         }
 
@@ -504,7 +522,7 @@ async fn test_outbound_failure() {
                 }) => {
                     assert!(inactive_ids.contains(&peer));
                 }
-                other => panic!("Unexpected Event: {:?}", other),
+                other => panic!("Unexpected behaviour event: {:?}.", other),
             }
         }
     };
